package templates

import scala.collection.mutable
import scala.util.{Success, Try}
import scala.xml.transform.{RewriteRule, RuleTransformer}
import scala.xml.{Node, NodeSeq, Text}

abstract class Template(protected[this] val source: Node) {
  protected[this] val msgNotFound = "Not found"
  private[this] val data = {
    val m:mutable.LinkedHashMap[Int, mutable.Queue[(String, String, mutable.LinkedHashMap[Int, (String, String)])]] =
      mutable.LinkedHashMap()
    val items = source \\ "_"

    for (i <- items) {
      if (i.attributes.nonEmpty || (i.child.length == 1)) {
        val idx = i.hashCode
        val attr: mutable.LinkedHashMap[Int, (String, String)] =
          if (i.attributes.nonEmpty) mutable.LinkedHashMap() else null
        val text = if (i.child.nonEmpty && !i.child.head.text.contains("\n")) i.child.head.text else null

        m.get(idx) match {
          case None =>
            m(idx) = mutable.Queue((s"tag: ${i.label}", text, attr))
          case _ =>
            m(idx).enqueue((s"tag: ${i.label}", text, attr))
        }

        if (i.attributes.nonEmpty) {
          for (a <- i.attributes) {
              m(idx).last._3(a.value.hashCode) = (a.key, a.value.text)
          }
        }
      }
    }

    m
  }

  protected[this] def get(content: Try[Node], attr: String = null): Text =
    content match {
        case Success(v) =>
          var res = Text(v.text)

          if (attr != null) v.attribute(attr) match {
            case None =>
              res = Text(msgNotFound)
            case o =>
              data(v.hashCode).head._3 -= o.get.hashCode
              res = Text(o.get.text)
          } else if (data(v.hashCode).nonEmpty) {
              if (data(v.hashCode).head._3 != null) {
                val e = data(v.hashCode).head.copy(_2 = null)

                data(v.hashCode).enqueue(e)
              }
            data(v.hashCode).dequeue()
          }

          res
        case _ => Text(msgNotFound)
  }

  def template: Node

  def render: Node = {
    val rule: RewriteRule = new RewriteRule {
      override def transform(n: Node): NodeSeq = n match {
        case v if v.attributes.nonEmpty && v.attributes.filter(_.value.text == msgNotFound).nonEmpty =>
          NodeSeq.Empty
        case v if v.child.length == 1 && v.child.head.text == msgNotFound =>
          NodeSeq.Empty
        case _ => n
      }
    }

    new RuleTransformer(rule).transform(template).head
  }

  def log(): Unit = data.foreach(e => println(e._2))
}
