package templates

import org.scalatest._
import xmlvalid._

import scala.xml.{Node, XML}

class RepoSpec extends WordSpec {
  "XMLValid for RTS example" should {
    "successfully loadFile(xmlPath) with well formed and valid xml" in {
      val validXml = XMLValid("src/test/resources/templates/rts/fpml-recordkeeping-merged-schema.xsd")
      val xml = validXml.loadFile("src/test/resources/templates/rts/Example repo RTS.xml")
      assert(xml.isInstanceOf[Node])
    }
  }

  it should {
    "throw exception in loadFile(xmlPath) with well formed but invalid xml" in {
      assertThrows[Exception] {
        val validXml = XMLValid("src/test/resources/templates/rts/fpml-recordkeeping-merged-schema.xsd")
        val xml = validXml.loadFile("src/test/resources/templates/rts/Example repo RTS invalid.xml")
      }
    }
  }

  "XMLValid for NSD example" should {
    "successfully loadFile(xmlPath) with well formed and valid xml" in {
      val validXml = XMLValid("src/test/resources/templates/nsd/nsd-ext-merged-schema.xsd")
      val xml = validXml.loadFile("src/test/resources/templates/nsd/Example repo NSD.xml")
      assert(xml.isInstanceOf[Node])
    }
  }

  it should {
    "throw exception in loadFile(xmlPath) with well formed but invalid xml" in {
      assertThrows[Exception] {
        val validXml = XMLValid("src/test/resources/templates/nsd/nsd-ext-merged-schema.xsd")
        val xml = validXml.loadFile("src/test/resources/templates/nsd/Example repo NSD invalid.xml")
      }
    }
  }


  "RepoTemplate" should {
    "generate valid RTS output xml using valid NSD source xml" ignore {
      val sourceXml: Node = XML.loadFile("src/test/resources/templates/nsd/Example repo NSD.xml")
      val outputXml = new RepoTemplate(sourceXml).template
      val checkedXml = XMLValid("src/test/resources/templates/rts/fpml-recordkeeping-merged-schema.xsd").loadString(outputXml.mkString)

      assert(checkedXml.isInstanceOf[Node])
    }
  }
}


